# encoding: UTF-8
require_relative 'certificate'
require_relative 'key_helper'

module SSLTool
  class PEMScanner
    attr_reader :pem_strings, :cert_strings, :key_strings, :garbage_strings
    RX_PEM_BLOCK = /(-----BEGIN.*?-----\n
                     (?:[A-Za-z0-9\+\/]+\n)*
                     [A-Za-z0-9\+\/]*
                     (?:=|==|=\n=)?
                     \n
                     -----END.*?-----
                   )/x

    class << self
      alias_method :[], :new
      alias_method :scan, :new

      def keys_from(s)
        scan(s).keys
      end

      def certs_from(s)
        scan(s).certs
      end
    end

    def initialize(s)
      s                              = s.dup.force_encoding('BINARY').gsub(/\r\n?/, "\n").gsub(/\s+\n/, "\n")
      @pem_strings, @garbage_strings = s.split(RX_PEM_BLOCK).map(&:strip).reject(&:empty?).partition { |s| s =~ RX_PEM_BLOCK }
      @cert_strings                  = @pem_strings.select { |s| s =~ /-----BEGIN CERTIFICATE-----/ }
      @key_strings                   = @pem_strings.select { |s| s =~ /-----BEGIN (EC |RSA )?PRIVATE KEY-----/ }
      @garbage_strings              += @pem_strings - @cert_strings - @key_strings
    end

    def certificates
      @certificates ||= cert_strings.map do |s|
        begin
          Certificate.new(s)
        rescue OpenSSL::X509::CertificateError
          garbage_strings << cert_strings.delete(s)
          nil
        end
      end.compact
    end
    alias_method :certs, :certificates

    def keys
      KeyHelper.keys(*key_strings)
    end

  end
end
